/*
 * common.h : definitions used system-wide
 *   by uratan! 2008.9.4
 */
#ifndef __COMMON_H__
#define __COMMON_H__

/* =============================================================== */
/*
 * system-wide macros and typedefs
 */
#define NULL    ((void *)0)

#define TRUE    1
#define FALSE   0

#define DONE    0
#define YET     1

typedef unsigned char  BYTE;
typedef unsigned short WORD;
typedef unsigned long  DWORD;
typedef unsigned int   BOOL;

typedef signed char    s_char;
typedef unsigned char  u_char;
typedef unsigned short u_short;
typedef unsigned long  u_long;

typedef void (*INTR_subFUNCs)(u_long x);

#define VOLATILE  volatile		/* enhance to look attractive */


/* =============================================================== */
/*
 * System configuration: Fosc, Fcclk, Fcco, Fpclk must be defined
 */
/*
 * Crystal frequence, 10MHz~25MHz should be the same as actual status.
 */
#define Fosc	12000000

/*
 * System frequence, should be (1~32)multiples of Fosc,
 * and should be equal or less than 60MHz.
 */
#define Fcclk	(Fosc * 2)		/* 24MHz */

/*
 * CCO frequence, should be 2/4/8/16 multiples of Fcclk,
 * ranged from 156MHz to 320MHz.
 */
#define Fcco	(Fcclk * 8)		/* 192MHz */

/*
 * VPB clock frequence, must be 1/2/4 multiples of (Fcclk / 4).
 */
#define Fpclk	(4 * (Fcclk / 4))		/* 24MHz */
	/*
	 * to use USB, Fpclk must be greater than 18MHz.
	 *   (otherwise, byte-lost is occured when reading from RX_DATA)
	 * you SHALL get "LPC214x User Manual Rev.02 - 25 July 2006".
	 * also you SHALL check both
	 *   http://www.standardics.nxp.com/microcontrollers/ and
	 *   http://www.nxp.com/
	 * to get more recent documents.
	 * (even so, these documents are SHIT !!)
	 */
	/*
	 * power consumptions by LPC-H2148 (OLIMEX)
	 *                                    (LEDs)          (level-converter of)
	 *            mode                     PWR   LINK  STAT uart0 : (from Vbus)
	 * --------------------------------------------------------------------
	 * when special
	 *   hard reset (hold)                  o     x     x     o   : 14.0mA
	 *   hard reset (hold)                  o     x     x     x   : 17.0mA
	 *   boot loader (idle)                 o     x     x     o   : 21.9mA
	 *   boot loader (idle)                 o     x     x     x   : 18.9mA
	 *
	 * at ArmUZK(A037+), when idle
	 *   ArmUZK(idle) 24MHz+24MHz(192MHz)   o     o     o     o   : 53mA
	 *   ArmUZK(idle) 48MHz+24MHz(192MHz)   o     o     o     o   : 70mA
	 *   ArmUZK(idle) 48MHz+48MHz(192MHz)   o     o     o     o   : 77mA
	 *   ArmUZK(idle) 60MHz+60MHz(240MHz)   o     o     o     o   : 90mA
	 *   ArmUZK(idle) 60MHz+30MHz(240MHz)   o     o     o     o   : 80mA
	 *   ArmUZK(idle) 36MHz+36MHz(288MHz)   o     o     o     o   : 68mA
	 *   ArmUZK(idle) 36MHz+18MHz(288MHz)   o     o     o     o   : 61mA
	 *
	 * at ArmUZK(A037+) 24MHz+24MHz
	 *   ArmUZK(idle) 24MHz+24MHz(192MHz)   o     o     o     x   : 49.7mA
	 *   ArmUZK(Sleep)                      o     o     x     x   : 15.8mA
	 *   ArmUZK(idle, BOGD+BORD)            o     o     o     x   : 47.9mA
	 *   ArmUZK(Sleep, BOGD+BORD)           o     o     x     x   : 15.7mA
	 *   ArmUZK(Sleep, BOGD+BORD+BODPDM)    o     o     x     x   : 15.6mA
	 *
	 * these may sink about
	 *   PWR_LED:         3mA
	 *   LINK_LED:        3mA
	 *   STAT_LED:        3mA  (including myLED: 1mA)
	 *   level-converter: 3mA
	 *   floating-I/O:    1mA
	 *   3.3V regulator   5mA
	 *   USB pull-up:     2mA + 1mA
	 */

/*
 * for software timer (loop by volatile u_long)
 */
#define LOOP_WAIT_1msec  2000		/* 1.1msec @ 24MHz+24MHz, -Os */


/* =============================================================== */
/*
 * use of I/O ports
 */
/*
 * use P0.7 for monitor LED
 */
#define LED_MON        (1<<7)
#define   MON_LED_OFF   FIO0SET = LED_MON
#define   MON_LED_ON    FIO0CLR = LED_MON

/*
 * use STAT LED(green, P1.24) for another monitor (LPC-H2148 only)
 */
#define LED_STAT         (1<<24)
#define   STAT_LED_OFF    FIO1SET = LED_STAT
#define   STAT_LED_ON     FIO1CLR = LED_STAT

/*
 * use P1.25, P1.26 for debug
 */
#define DEBUG_PORT         (1<<25)
#define   SET_DEBUG_PORT    FIO1SET = DEBUG_PORT
#define   CLR_DEBUG_PORT    FIO1CLR = DEBUG_PORT
#define DEBUG_PORT2        (1<<26)
#define   SET_DEBUG_PORT2   FIO1SET = DEBUG_PORT2
#define   CLR_DEBUG_PORT2   FIO1CLR = DEBUG_PORT2

/* =============================================================== */
/*
 * register bitmaps
 */
/*
 * PCONP: Power Control for Peripherals register
 */
		/* unused      (1<< 0) */
#define PP_PCTIM0      (1<< 1)
#define PP_PCTIM1      (1<< 2)
#define PP_PCUART0     (1<< 3)
#define PP_PCUART1     (1<< 4)
#define PP_PCPWM0      (1<< 5)
		/* unused      (1<< 6) */
#define PP_PCI2C0      (1<< 7)
#define PP_PCSPI0      (1<< 8)
#define PP_PCRTC       (1<< 9)
#define PP_PCSPI1      (1<<10)
		/* unused      (1<<11) */
#define PP_PCAD0       (1<<12)
		/* unused      (1<<13) */
					/*   ...   */
		/* unused      (1<<18) */
#define PP_PCI2C1      (1<<19)
#define PP_PCAD1       (1<<20)
		/* unused      (1<<21) */
					/*   ...   */
		/* unused      (1<<30) */
#define PP_PUSB        (1<<31)

/* =============================================================== */
/*
 * shared functions
 */
/* in common.c */
extern void cm_sys_pll_init(void);
extern void cm_usb_pll_init(void);
extern void cm_init_serial0(u_long baudrate);
extern void cm_putchar(int ch);
extern int cm_keyin(void);
extern u_char cm_getchar(void);
/* in rprintf.c */
extern void rprintf(char const *format, ...);
/* in swi_handler.S */
extern u_long IntGetCPSR(void);
extern u_long IntDisable(void);
extern u_long IntEnable(void);
extern void IntRestore(u_long oldstate);
extern u_long FiqDisable(void);
extern u_long FiqEnable(void);
extern void FiqRestore(u_long oldstate);


#endif  /* __COMMON_H__ */
